/* -*- Mode: IDL; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * CORBA interface for the Evolution shell.
 *
 * Authors:
 *   Ettore Perazzoli <ettore@ximian.com>
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 */

#ifndef _GNOME_EVOLUTION_IDL
#define _GNOME_EVOLUTION_IDL

#include <Bonobo.idl>

#include <Evolution-ConfigControl.idl>
#include <Evolution-Shell.idl>
#include <Evolution-Component.idl>

#endif
