/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the program; if not, see <http://www.gnu.org/licenses/>  
 *
 *
 * Authors:
 *		Ettore Perazzoli <ettore@ximian.com>
 *
 * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
 *
 */

#ifndef E_SHELL_CONSTANTS_H
#define E_SHELL_CONSTANTS_H

#define E_SHELL_URI_PREFIX		"evolution:"
#define E_SHELL_URI_PREFIX_LEN		10

#define E_SHELL_DEFAULTURI_PREFIX	"default:"
#define E_SHELL_DEFAULTURI_PREFIX_LEN	8

#define E_SHELL_MINI_ICON_SUFFIX	"-mini"
#define E_SHELL_MINI_ICON_SUFFIX_LEN	5

#define E_SHELL_ICON_SIZE		48
#define E_SHELL_MINI_ICON_SIZE		16

#define E_PATH_SEPARATOR                '/'
#define E_PATH_SEPARATOR_S		"/"

#define E_LOCAL_STORAGE_NAME            "local"
#define E_SUMMARY_STORAGE_NAME          "summary"

#define E_SUMMARY_URI			"evolution:/summary"
#define E_LOCAL_INBOX_URI		"evolution:/local/Inbox"
#define E_LOCAL_CONTACTS_URI		"evolution:/local/Contacts"
#define E_LOCAL_CALENDAR_URI		"evolution:/local/Calendar"
#define E_LOCAL_TASKS_URI		"evolution:/local/Tasks"

#endif
