/* XPM */
static char *ximian.t[] = {
/* columns rows colors chars-per-pixel */
"48 48 235 2",
"   c #000000",
".  c #090604",
"X  c #080708",
"o  c #0e0a01",
"O  c #0d0a0a",
"+  c #100d0e",
"@  c #120f10",
"#  c #1c1402",
"$  c #1e1905",
"%  c #1e1908",
"&  c #151213",
"*  c #181315",
"=  c #1b1618",
"-  c #1e1b10",
";  c #1d181a",
":  c #221201",
">  c #271902",
",  c #2c1a01",
"<  c #311d02",
"1  c #2c2204",
"2  c #2d260c",
"3  c #2c2410",
"4  c #2e2914",
"5  c #2e2918",
"6  c #332002",
"7  c #3f2503",
"8  c #3c2e05",
"9  c #3d3005",
"0  c #3e3209",
"q  c #3d3311",
"w  c #3b3218",
"e  c #2f2c22",
"r  c #3f3b2c",
"t  c #452502",
"y  c #4b2c03",
"u  c #453005",
"i  c #45320b",
"p  c #483506",
"a  c #493913",
"s  c #512b02",
"d  c #583504",
"f  c #5c3b05",
"g  c #5b3f17",
"h  c #613704",
"j  c #683703",
"k  c #4d431d",
"l  c #56400d",
"z  c #584307",
"x  c #5c4b10",
"c  c #59491b",
"v  c #4e472c",
"b  c #5d5021",
"n  c #5d532d",
"m  c #674005",
"M  c #684f09",
"N  c #6b540a",
"B  c #6c5a1a",
"V  c #774c07",
"C  c #7d4a06",
"Z  c #725308",
"A  c #7f5012",
"S  c #6c5f2c",
"D  c #7b610c",
"F  c #7b6312",
"G  c #7b661a",
"H  c #6d6133",
"J  c #6d633a",
"K  c #7c6923",
"L  c #7c6b2d",
"P  c #7c6c30",
"I  c #7d703e",
"U  c #5e5840",
"Y  c #7d7245",
"T  c #7e785e",
"R  c #8b4a04",
"E  c #825207",
"W  c #965004",
"Q  c #8b6210",
"!  c #896e13",
"~  c #936c0b",
"^  c #95710d",
"/  c #95741a",
"(  c #896f20",
")  c #8c7a36",
"_  c #a25605",
"`  c #ae5c05",
"'  c #a16b0a",
"]  c #aa6107",
"[  c #ae6608",
"{  c #ab6f0a",
"}  c #a57f0e",
"|  c #ae730b",
" . c #ac7d0d",
".. c #b16c09",
"X. c #ba6406",
"o. c #be6906",
"O. c #b1780c",
"+. c #bd7f12",
"@. c #8c7d42",
"#. c #c06c07",
"$. c #c26e08",
"%. c #c57308",
"&. c #c57e0c",
"*. c #c87709",
"=. c #cb7b0a",
"-. c #9a8125",
";. c #9b832c",
":. c #a6800f",
">. c #a8840f",
",. c #aa891b",
"<. c #bc830d",
"1. c #be9123",
"2. c #b99a2a",
"3. c #ba9d33",
"4. c #ba9f3b",
"5. c #baa03e",
"6. c #9c8a48",
"7. c #9c8d54",
"8. c #9c8f5a",
"9. c #9c905a",
"0. c #8e8563",
"q. c #9e9676",
"w. c #baa141",
"e. c #bba758",
"r. c #bba85c",
"t. c #ada377",
"y. c #bcb07d",
"u. c #ce800b",
"i. c #cd8b0e",
"p. c #c38a1a",
"a. c #cb8913",
"s. c #c19111",
"d. c #c7921c",
"f. c #c59a11",
"g. c #c59c1f",
"h. c #cc9814",
"j. c #d1850c",
"k. c #d58c0d",
"l. c #d88f0e",
"z. c #d08512",
"x. c #d0900e",
"c. c #da930e",
"v. c #dd980f",
"b. c #d39510",
"n. c #d09d12",
"m. c #d59b1d",
"M. c #dd9b10",
"N. c #c8a016",
"B. c #d4a513",
"V. c #daa111",
"C. c #c9a62b",
"Z. c #c9ab3b",
"A. c #d8b024",
"S. c #d9b73a",
"D. c #e09d10",
"F. c #e3a311",
"G. c #e6a812",
"H. c #e6a818",
"J. c #e9ad13",
"K. c #e9ae19",
"L. c #e4b315",
"P. c #e3b21b",
"I. c #e7b91d",
"U. c #edb314",
"Y. c #edb51b",
"T. c #f0b715",
"R. c #f1bb15",
"E. c #e2b328",
"W. c #eab921",
"Q. c #c9ad41",
"!. c #caae49",
"~. c #cab050",
"^. c #dabf5c",
"/. c #cbb86d",
"(. c #cbb971",
"). c #cbbc79",
"_. c #f5c117",
"`. c #f6c41a",
"'. c #e9c33e",
"]. c #f7c622",
"[. c #f7c826",
"{. c #f7c92b",
"}. c #fac926",
"|. c #fbcb2b",
" X c #f7cc33",
".X c #f7cf3c",
"XX c #f8cc33",
"oX c #f8ce3a",
"OX c #f9d03c",
"+X c #e9c647",
"@X c #e8c749",
"#X c #e8c84d",
"$X c #e9cc5e",
"%X c #f8d143",
"&X c #f8d34c",
"*X c #f8d453",
"=X c #f9d65a",
"-X c #f9d85c",
";X c #eacf68",
":X c #ead273",
">X c #ead479",
",X c #fed76c",
"<X c #f9d964",
"1X c #f9db6b",
"2X c #f9dd73",
"3X c #fcdb7d",
"4X c #fae17d",
"5X c #bdb387",
"6X c #ccbf87",
"7X c #ccc08a",
"8X c #ebd98d",
"9X c #fddc86",
"0X c #fae282",
"qX c #fae48b",
"wX c #fbe593",
"eX c #fbe599",
"rX c #fbe89d",
"tX c #eee4ba",
"yX c #fde5a4",
"uX c #fee7ac",
"iX c #fbeaa3",
"pX c #fbebaa",
"aX c #fceeb3",
"sX c #fdedba",
"dX c #feefc8",
"fX c #fdf2c3",
"gX c #fef4ce",
"hX c #fef6d3",
"jX c #fff5db",
"kX c #fef8d8",
"lX c #fff6e1",
"zX c #fffae3",
"xX c #fff9eb",
"cX c #fefbf3",
"vX c None",
/* pixels */
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX                  vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX                              vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvX                                          vXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvX                0 0 D D D 9 1                 vXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvX            % B 2. X X X{.[.`.`.`.R.:.M o             vXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvX  X       % ;.%X%X%X%X%XoX X{.[.`.`._.R.R.U.~ #           vXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvX  &     o K &X*X*X=X=X*X&X&X%XoX X{.].`._.R.T.U.V.Z           vXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvX  &     2 Q.2XpXeX$X6.<X-X-X*X*X%XOX X{.`.`._.R.U.J.G.<.#         vXvXvXvXvXvXvXvX",
"vXvXvXvXvXvX  *     2 #XpXcXcXqXn :X4X2X1X<X-X*X&X.X X[.`.`.R.R.U.J.G.b.>         vXvXvXvXvXvXvX",
"vXvXvXvXvX  &     2 #XaXxXcXeXI 9.0X0X0X4X2X1X-X*X%XoX X].`._.R.U.U.G.F.x.>         vXvXvXvXvXvX",
"vXvXvXvXvXO     2 @XwXjXlXpX/.r wXwXwXqXqX0X2X1X-X5.S. X{.`.`.R.U.U.G.F.D.i.o       vXvXvXvXvXvX",
"vXvXvXvX  O     Z.2XdXdXsX9X5 y.iXpXiXrX).8.H 4     q .X{.].`._.R.U.J.G.v.v.^         vXvXvXvXvX",
"vXvXvX  O     K *XyXsXsXwX8.r pXaXaXaXt.      r J L   -. X[.`._.R.U.J.G.D.v.l.f         vXvXvXvX",
"vXvXvX  X   % %X2XyXuXwXqX- 5XsXfXfXfXT   J wX0X2X<Xb % .X[.`._.R.U.J.G.F.v.l.u.o       vXvXvXvX",
"vXvXvXX     -.%X9XwXeX9X8.e aXfXhXkXhXtXq.7XwX0X2X<Xw.  G {.`._.R.U.U.G.D.v.c.j.j       vXvXvXvX",
"vXvX  O   o XX*X3X9X4X0Xv 0.aXfXhXzXkXfXaXpXwX0X2X1X-X2 o A.`.`._.U.U.G.F.v.k.j.%.        vXvXvX",
"vXvX      x XX=X,X2X2X0X  7XaXfXhXhXhXfXaXiXwX0X4X<X*X;.  0 `.`.R.U.U.F.F.v.c.k.u.7       vXvXvX",
"vXvX.     ,.XX*X=X<X1Xr.  rXpXaXfXfXfXaXpXrXqX0X1X~.5.'.o   D `.R.T.U.G.D.v.l.j.u.C       vXvXvX",
"vX        W.XXOX&X=X1Xe.  8XrXpXaXaXaXpXrXwXqXI o             >.T.U.J.G.D.v.k.j.=.[         vXvX",
"vX      % `.|.OX%X*X<X;X  Y wXrXiXiXiXrXwX(.4                 %   l M.G.D.c.k.j.=.*.        vXvX",
"vX      9 `.}.XX%X&X=X<XS   7.qXqXwXwXqX7.                          1 V.D.c.k.j.*.$.<       vXvX",
"vX      9 _.].|.oX%X&X-X$X4   J >X0X0X6.                              d.v.l.k.=.*.$.,       vXvX",
"vX      8 _._.[.|.oX%X*X=X<XS   O n 6.          + k P 2     4 o     3 D.v.l.k.=.*.$.,       vXvX",
"vX      8 _._.`.{. X.X%X&X*X=X^.n           + ) '. X{.I.    g.^   w m.D.c.k.j.=.%.o.,       vXvX",
"vX      8 R._._.`.[.{..X%X%X&X*X*X!.        +X.X X[.`._.    U.U.h.H.D.M.l.j.=.=.$.o.:       vXvX",
"vX      o R.R._._.`.[.{. X.X.X%X%X5.        -.{.[.`.`._.    U.J.F.F.v.c.k.j.=.%.#.o.        vXvX",
"vX        h.T.R._.`.`.`.[.{.{. X{.2.  L 2   o I.`.`._.L.  4 U.G.F.D.v.k.k.=.*.$.o.W         vXvX",
"vXvX      ^ U.R.R._._._.`.`.[.[.[.F   C.W.1   D _._.R.:.  % M.H.D.v.c.k.j.=.%.#.X.j       vXvXvX",
"vXvX      p J.U.U.R.R._._.`.`.`.N.9 - `.`.L.> o L.T.R.s.2   > b.v.v.k.j.=.*.$.o.X.:   +   vXvXvX",
"vXvX        M.J.U.U.R.R.B.f.L.N   o ! `.>.o   q A.U.J.J.M.l   > <.| V =.*.*.#.X._   O @   vXvXvX",
"vXvXvX      Z V.J.J.J.U.z o 1 - ( L.R.f.o   c Y.U.U.J.J.V.F.Q o o V 6 ..%.#.X.X.t   * . vXvXvXvX",
"vXvXvX      o c.V.J.J.b.  % / W.U.U.U.8   c Y.J.J.G.F.F.M.v.c.+.>     A #.o.X._   X =   vXvXvXvX",
"vXvXvX        u v.F.F. .1.K.J.J.K.U.~   c K.G.G.F.F.v.v.c.c.k.j.z.d g ] o.X.X.>   = X   vXvXvXvX",
"vXvXvXvX        ~ v.v.D.F.F.F.G.G.,   a H.F.F.D.v.v.v.v.l.k.j.u.*.*.$.o.X.X.j   @ *   vXvXvXvXvX",
"vXvXvXvXvX      o O.v.v.v.D.D.D.D.  3 m.M.D.M.v.v.c.l.k.j.j.=.*.%.#.o.X.X.W   X ;   vXvXvXvXvXvX",
"vXvXvXvXvX        o | l.c.c.v.c.v.i p.c.c.c.c.l.l.k.j.j.u.=.*.%.#.o.X.X.W . . * X   vXvXvXvXvXvX",
"vXvXvXvXvXvX        o { k.k.k.k.l.a.l.l.k.k.k.k.j.u.u.*.*.*.#.o.o.X.X.W o   * +   vXvXvXvXvXvXvX",
"vXvXvXvXvXvXvX        o E j.j.j.k.j.j.k.j.j.j.=.=.*.*.*.$.#.o.X.X.X.j     @ @   vXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvX          < ..=.=.=.=.=.=.*.*.*.*.%.$.#.o.X.X.X.W ,   X * O   vXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvX            y [ *.*.*.%.%.$.#.#.o.o.X.X.X.W t     O & X   vXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvX              : h R o.#.o.X.X.X.` R s :     O @ +     vXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvX                    , , ,         . O + + .   vXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvX                    .   . X X O O X       vXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX            X X X X           vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX                  vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX",
"vXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvXvX"
};
