EXPORTS
	e_composer_header_table_set_account_name
	e_composer_header_table_set_destinations_bcc
	e_composer_header_table_set_destinations_to
	e_composer_header_table_set_subject
	e_composer_header_table_set_subject
	e_msg_composer_attach
	e_msg_composer_get_header_table
	e_msg_composer_new
	e_msg_composer_send
	e_msg_composer_set_body
	e_msg_composer_set_body_text
	em_composer_utils_setup_callbacks
	em_folder_selection_button_get_selection
	em_folder_selection_button_new
	em_folder_selection_button_set_selection
	em_popup_new
	em_popup_target_new_attachments
	em_utils_prompt_user
	mail_component_get_folder
	mail_component_get_folder_uri
	mail_component_peek_base_directory
	mail_msg_fast_ordered_push
	mail_msg_free
	mail_msg_new
	mail_msg_unref
	mail_tool_uri_to_folder
	mail_win32_get_mail_thread_queued
