EXPORTS
	e_calendar_view_get_calendar
	e_calendar_view_get_selected_time_range
	e_calendar_view_get_type
	e_calendar_view_new_appointment
	e_calendar_view_new_appointment_for
	e_day_view_convert_time_to_display
	e_day_view_ensure_rows_visible
	e_day_view_find_event_from_item
	e_day_view_get_event_position
	e_day_view_get_long_event_position
	e_day_view_get_type
	e_day_view_main_item_get_type
	e_day_view_top_item_get_day_label
	e_day_view_update_calendar_selection_time
	e_week_view_find_event_from_item
	e_week_view_get_span_position
	e_week_view_get_type
	e_week_view_jump_to_button_item
	e_week_view_main_item_get_type
	gnome_calendar_get_current_view_widget
	gnome_calendar_get_e_calendar_widget
	gnome_calendar_get_search_bar_widget
	gnome_calendar_get_task_pad
	gnome_calendar_get_timezone
	gnome_calendar_get_type
	gnome_calendar_get_view
	gnome_calendar_get_view_notebook_widget
	gnome_calendar_get_visible_time_range
	goto_dialog
