/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the program; if not, see <http://www.gnu.org/licenses/>  
 *
 *
 * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
 *
 */
/*
 * These are the timezone names from the Olson timezone data.
 * We only place them here so gettext picks them up for translation.
 * Don't include in any C files.
 */

N_("Africa/Abidjan");
N_("Africa/Accra");
N_("Africa/Addis_Ababa");
N_("Africa/Algiers");
N_("Africa/Asmera");
N_("Africa/Bamako");
N_("Africa/Bangui");
N_("Africa/Banjul");
N_("Africa/Bissau");
N_("Africa/Blantyre");
N_("Africa/Brazzaville");
N_("Africa/Bujumbura");
N_("Africa/Cairo");
N_("Africa/Casablanca");
N_("Africa/Ceuta");
N_("Africa/Conakry");
N_("Africa/Dakar");
N_("Africa/Dar_es_Salaam");
N_("Africa/Djibouti");
N_("Africa/Douala");
N_("Africa/El_Aaiun");
N_("Africa/Freetown");
N_("Africa/Gaborone");
N_("Africa/Harare");
N_("Africa/Johannesburg");
N_("Africa/Kampala");
N_("Africa/Khartoum");
N_("Africa/Kigali");
N_("Africa/Kinshasa");
N_("Africa/Lagos");
N_("Africa/Libreville");
N_("Africa/Lome");
N_("Africa/Luanda");
N_("Africa/Lubumbashi");
N_("Africa/Lusaka");
N_("Africa/Malabo");
N_("Africa/Maputo");
N_("Africa/Maseru");
N_("Africa/Mbabane");
N_("Africa/Mogadishu");
N_("Africa/Monrovia");
N_("Africa/Nairobi");
N_("Africa/Ndjamena");
N_("Africa/Niamey");
N_("Africa/Nouakchott");
N_("Africa/Ouagadougou");
N_("Africa/Porto-Novo");
N_("Africa/Sao_Tome");
N_("Africa/Timbuktu");
N_("Africa/Tripoli");
N_("Africa/Tunis");
N_("Africa/Windhoek");
N_("America/Adak");
N_("America/Anchorage");
N_("America/Anguilla");
N_("America/Antigua");
N_("America/Araguaina");
N_("America/Aruba");
N_("America/Asuncion");
N_("America/Barbados");
N_("America/Belem");
N_("America/Belize");
N_("America/Boa_Vista");
N_("America/Bogota");
N_("America/Boise");
N_("America/Buenos_Aires");
N_("America/Cambridge_Bay");
N_("America/Cancun");
N_("America/Caracas");
N_("America/Catamarca");
N_("America/Cayenne");
N_("America/Cayman");
N_("America/Chicago");
N_("America/Chihuahua");
N_("America/Cordoba");
N_("America/Costa_Rica");
N_("America/Cuiaba");
N_("America/Curacao");
N_("America/Danmarkshavn");
N_("America/Dawson");
N_("America/Dawson_Creek");
N_("America/Denver");
N_("America/Detroit");
N_("America/Dominica");
N_("America/Edmonton");
N_("America/Eirunepe");
N_("America/El_Salvador");
N_("America/Fortaleza");
N_("America/Glace_Bay");
N_("America/Godthab");
N_("America/Goose_Bay");
N_("America/Grand_Turk");
N_("America/Grenada");
N_("America/Guadeloupe");
N_("America/Guatemala");
N_("America/Guayaquil");
N_("America/Guyana");
N_("America/Halifax");
N_("America/Havana");
N_("America/Hermosillo");
N_("America/Indiana/Indianapolis");
N_("America/Indiana/Knox");
N_("America/Indiana/Marengo");
N_("America/Indiana/Vevay");
N_("America/Indianapolis");
N_("America/Inuvik");
N_("America/Iqaluit");
N_("America/Jamaica");
N_("America/Jujuy");
N_("America/Juneau");
N_("America/Kentucky/Louisville");
N_("America/Kentucky/Monticello");
N_("America/La_Paz");
N_("America/Lima");
N_("America/Los_Angeles");
N_("America/Louisville");
N_("America/Maceio");
N_("America/Managua");
N_("America/Manaus");
N_("America/Martinique");
N_("America/Mazatlan");
N_("America/Mendoza");
N_("America/Menominee");
N_("America/Merida");
N_("America/Mexico_City");
N_("America/Miquelon");
N_("America/Monterrey");
N_("America/Montevideo");
N_("America/Montreal");
N_("America/Montserrat");
N_("America/Nassau");
N_("America/New_York");
N_("America/Nipigon");
N_("America/Nome");
N_("America/Noronha");
N_("America/North_Dakota/Center");
N_("America/Panama");
N_("America/Pangnirtung");
N_("America/Paramaribo");
N_("America/Phoenix");
N_("America/Port-au-Prince");
N_("America/Port_of_Spain");
N_("America/Porto_Velho");
N_("America/Puerto_Rico");
N_("America/Rainy_River");
N_("America/Rankin_Inlet");
N_("America/Recife");
N_("America/Regina");
N_("America/Rio_Branco");
N_("America/Rosario");
N_("America/Santiago");
N_("America/Santo_Domingo");
N_("America/Sao_Paulo");
N_("America/Scoresbysund");
N_("America/Shiprock");
N_("America/St_Johns");
N_("America/St_Kitts");
N_("America/St_Lucia");
N_("America/St_Thomas");
N_("America/St_Vincent");
N_("America/Swift_Current");
N_("America/Tegucigalpa");
N_("America/Thule");
N_("America/Thunder_Bay");
N_("America/Tijuana");
N_("America/Tortola");
N_("America/Vancouver");
N_("America/Whitehorse");
N_("America/Winnipeg");
N_("America/Yakutat");
N_("America/Yellowknife");
N_("Antarctica/Casey");
N_("Antarctica/Davis");
N_("Antarctica/DumontDUrville");
N_("Antarctica/Mawson");
N_("Antarctica/McMurdo");
N_("Antarctica/Palmer");
N_("Antarctica/South_Pole");
N_("Antarctica/Syowa");
N_("Antarctica/Vostok");
N_("Arctic/Longyearbyen");
N_("Asia/Aden");
N_("Asia/Almaty");
N_("Asia/Amman");
N_("Asia/Anadyr");
N_("Asia/Aqtau");
N_("Asia/Aqtobe");
N_("Asia/Ashgabat");
N_("Asia/Baghdad");
N_("Asia/Bahrain");
N_("Asia/Baku");
N_("Asia/Bangkok");
N_("Asia/Beirut");
N_("Asia/Bishkek");
N_("Asia/Brunei");
N_("Asia/Calcutta");
N_("Asia/Choibalsan");
N_("Asia/Chongqing");
N_("Asia/Colombo");
N_("Asia/Damascus");
N_("Asia/Dhaka");
N_("Asia/Dili");
N_("Asia/Dubai");
N_("Asia/Dushanbe");
N_("Asia/Gaza");
N_("Asia/Harbin");
N_("Asia/Hong_Kong");
N_("Asia/Hovd");
N_("Asia/Irkutsk");
N_("Asia/Istanbul");
N_("Asia/Jakarta");
N_("Asia/Jayapura");
N_("Asia/Jerusalem");
N_("Asia/Kabul");
N_("Asia/Kamchatka");
N_("Asia/Karachi");
N_("Asia/Kashgar");
N_("Asia/Katmandu");
N_("Asia/Krasnoyarsk");
N_("Asia/Kuala_Lumpur");
N_("Asia/Kuching");
N_("Asia/Kuwait");
N_("Asia/Macao");
N_("Asia/Macau");
N_("Asia/Magadan");
N_("Asia/Makassar");
N_("Asia/Manila");
N_("Asia/Muscat");
N_("Asia/Nicosia");
N_("Asia/Novosibirsk");
N_("Asia/Omsk");
N_("Asia/Oral");
N_("Asia/Phnom_Penh");
N_("Asia/Pontianak");
N_("Asia/Pyongyang");
N_("Asia/Qatar");
N_("Asia/Qyzylorda");
N_("Asia/Rangoon");
N_("Asia/Riyadh");
N_("Asia/Saigon");
N_("Asia/Sakhalin");
N_("Asia/Samarkand");
N_("Asia/Seoul");
N_("Asia/Shanghai");
N_("Asia/Singapore");
N_("Asia/Taipei");
N_("Asia/Tashkent");
N_("Asia/Tbilisi");
N_("Asia/Tehran");
N_("Asia/Thimphu");
N_("Asia/Tokyo");
N_("Asia/Ujung_Pandang");
N_("Asia/Ulaanbaatar");
N_("Asia/Urumqi");
N_("Asia/Vientiane");
N_("Asia/Vladivostok");
N_("Asia/Yakutsk");
N_("Asia/Yekaterinburg");
N_("Asia/Yerevan");
N_("Atlantic/Azores");
N_("Atlantic/Bermuda");
N_("Atlantic/Canary");
N_("Atlantic/Cape_Verde");
N_("Atlantic/Faeroe");
N_("Atlantic/Jan_Mayen");
N_("Atlantic/Madeira");
N_("Atlantic/Reykjavik");
N_("Atlantic/South_Georgia");
N_("Atlantic/St_Helena");
N_("Atlantic/Stanley");
N_("Australia/Adelaide");
N_("Australia/Brisbane");
N_("Australia/Broken_Hill");
N_("Australia/Darwin");
N_("Australia/Hobart");
N_("Australia/Lindeman");
N_("Australia/Lord_Howe");
N_("Australia/Melbourne");
N_("Australia/Perth");
N_("Australia/Sydney");
N_("Europe/Amsterdam");
N_("Europe/Andorra");
N_("Europe/Athens");
N_("Europe/Belfast");
N_("Europe/Belgrade");
N_("Europe/Berlin");
N_("Europe/Bratislava");
N_("Europe/Brussels");
N_("Europe/Bucharest");
N_("Europe/Budapest");
N_("Europe/Chisinau");
N_("Europe/Copenhagen");
N_("Europe/Dublin");
N_("Europe/Gibraltar");
N_("Europe/Helsinki");
N_("Europe/Istanbul");
N_("Europe/Kaliningrad");
N_("Europe/Kiev");
N_("Europe/Lisbon");
N_("Europe/Ljubljana");
N_("Europe/London");
N_("Europe/Luxembourg");
N_("Europe/Madrid");
N_("Europe/Malta");
N_("Europe/Minsk");
N_("Europe/Monaco");
N_("Europe/Moscow");
N_("Europe/Nicosia");
N_("Europe/Oslo");
N_("Europe/Paris");
N_("Europe/Prague");
N_("Europe/Riga");
N_("Europe/Rome");
N_("Europe/Samara");
N_("Europe/San_Marino");
N_("Europe/Sarajevo");
N_("Europe/Simferopol");
N_("Europe/Skopje");
N_("Europe/Sofia");
N_("Europe/Stockholm");
N_("Europe/Tallinn");
N_("Europe/Tirane");
N_("Europe/Uzhgorod");
N_("Europe/Vaduz");
N_("Europe/Vatican");
N_("Europe/Vienna");
N_("Europe/Vilnius");
N_("Europe/Warsaw");
N_("Europe/Zagreb");
N_("Europe/Zaporozhye");
N_("Europe/Zurich");
N_("Indian/Antananarivo");
N_("Indian/Chagos");
N_("Indian/Christmas");
N_("Indian/Cocos");
N_("Indian/Comoro");
N_("Indian/Kerguelen");
N_("Indian/Mahe");
N_("Indian/Maldives");
N_("Indian/Mauritius");
N_("Indian/Mayotte");
N_("Indian/Reunion");
N_("Pacific/Apia");
N_("Pacific/Auckland");
N_("Pacific/Chatham");
N_("Pacific/Easter");
N_("Pacific/Efate");
N_("Pacific/Enderbury");
N_("Pacific/Fakaofo");
N_("Pacific/Fiji");
N_("Pacific/Funafuti");
N_("Pacific/Galapagos");
N_("Pacific/Gambier");
N_("Pacific/Guadalcanal");
N_("Pacific/Guam");
N_("Pacific/Honolulu");
N_("Pacific/Johnston");
N_("Pacific/Kiritimati");
N_("Pacific/Kosrae");
N_("Pacific/Kwajalein");
N_("Pacific/Majuro");
N_("Pacific/Marquesas");
N_("Pacific/Midway");
N_("Pacific/Nauru");
N_("Pacific/Niue");
N_("Pacific/Norfolk");
N_("Pacific/Noumea");
N_("Pacific/Pago_Pago");
N_("Pacific/Palau");
N_("Pacific/Pitcairn");
N_("Pacific/Ponape");
N_("Pacific/Port_Moresby");
N_("Pacific/Rarotonga");
N_("Pacific/Saipan");
N_("Pacific/Tahiti");
N_("Pacific/Tarawa");
N_("Pacific/Tongatapu");
N_("Pacific/Truk");
N_("Pacific/Wake");
N_("Pacific/Wallis");
N_("Pacific/Yap");
