/* Automatically generated. Do not edit. */
char *s = N_("Adjust Score");
char *s = N_("Assign Color");
char *s = N_("Assign Score");
char *s = N_("Attachments");
char *s = N_("BCC");
char *s = N_("Beep");
char *s = N_("CC");
char *s = N_("Completed On");
char *s = N_("contains");
char *s = N_("Copy to Folder");
char *s = N_("Date received");
char *s = N_("Date sent");
char *s = N_("Delete");
char *s = N_("Deleted");
char *s = N_("does not contain");
char *s = N_("does not end with");
char *s = N_("does not exist");
char *s = N_("does not return");
char *s = N_("does not sound like");
char *s = N_("does not start with");
char *s = N_("Do Not Exist");
char *s = N_("Draft");
char *s = N_("ends with");
char *s = N_("Exist");
char *s = N_("exists");
char *s = N_("Expression");
char *s = N_("Follow Up");
char *s = N_("Important");
char *s = N_("is");
char *s = N_("is after");
char *s = N_("is before");
char *s = N_("is Flagged");
char *s = N_("is greater than");
char *s = N_("is less than");
char *s = N_("is not");
char *s = N_("is not Flagged");
char *s = N_("is not set");
char *s = N_("is set");
char *s = N_("Junk");
char *s = N_("Junk Test");
char *s = N_("Label");
char *s = N_("Mailing list");
char *s = N_("Match All");
char *s = N_("Message Body");
char *s = N_("Message Header");
char *s = N_("Message is Junk");
char *s = N_("Message is not Junk");
char *s = N_("Move to Folder");
char *s = N_("Pipe to Program");
char *s = N_("Play Sound");
/* Translators: "Read" as in "has been read" (em-filter-i18n.h) */
char *s = N_("Read");
char *s = N_("Recipients");
char *s = N_("Regex Match");
char *s = N_("Replied to");
char *s = N_("returns");
char *s = N_("returns greater than");
char *s = N_("returns less than");
char *s = N_("Run Program");
char *s = N_("Score");
char *s = N_("Sender");
char *s = N_("Set Label");
char *s = N_("Set Status");
char *s = N_("Size (kB)");
char *s = N_("sounds like");
char *s = N_("Source Account");
char *s = N_("Specific header");
char *s = N_("starts with");
char *s = N_("Status");
char *s = N_("Stop Processing");
char *s = N_("Subject");
char *s = N_("Unset Status");
