EXPORTS
	e_cell_height
	e_cell_popup_get_type
	e_cell_text_copy_clipboard
	e_cell_text_delete_selection
	e_cell_text_get_selection
	e_cell_text_get_text_by_view
	e_cell_text_get_type
	e_cell_text_paste_clipboard
	e_cell_text_set_selection
	e_cell_text_set_value
	e_cell_tree_view_get_subcell_view
	e_table_click_to_add_get_type
	e_table_col_get_type
	e_table_get_cell_at
	e_table_get_cell_geometry
	e_table_get_type
	e_table_group_container_get_type
	e_table_group_get_type
	e_table_group_leaf_get_type
	e_table_header_get_column
	e_table_header_get_columns
	e_table_header_item_get_type
	e_table_item_enter_edit
	e_table_item_get_cell_geometry
	e_table_item_get_type
	e_table_model_column_count
	e_table_model_is_cell_editable
	e_table_model_value_at
	e_table_subset_get_type
	e_tree_get_cell_at
	e_tree_get_cell_geometry
	e_tree_get_header_item
	e_tree_get_item
	e_tree_get_model
	e_tree_get_type
	e_tree_model_node_is_expandable
	e_tree_table_adapter_node_is_expanded
	e_tree_table_adapter_node_set_expanded
	ethi_change_sort_state
